@@ -3,7 +3,7 @@ import typing
 import time
 from typing import Any, Coroutine
 from opentrons import hardware_control
-from opentrons.protocol_api import ProtocolContext
+from opentrons.protocol_api import ProtocolContext, InstrumentContext
 from opentrons.protocol_api.labware import Well
 from opentrons.types import Point, Mount
 from pydantic import BaseModel
@@ -35,7 +35,8 @@ def read_offsets_file():
         with open(offset_file) as json_file:
             offsets = json.load(json_file)
     except FileNotFoundError:
-        offsets = {'x': 11.3, 'y': 4.3, 'z': 0.38}
+        print("offsets file not found at " + offset_file)
+        offsets = {'x': 11.3, 'y': 4.3, 'z': 0.38, "100mw_calibration": 13.88}
     return offsets
 
 
@@ -58,13 +59,7 @@ class Controller(BaseModel):
     z_min = 60  # generally should have to go lower than this
     z_max = 241.0
     z_min_buffer = 20.0
-
-    ###vvvvvvv#####
-    z_offset_where_100mw_cm_2_in_mm = 13.88
-
-    # from this data
-    # https://docs.google.com/spreadsheets/d/164BT17Gjqano3Rzso_YlVfvEHDgdcG7_JK_o8RAbTjg/edit?usp=sharing
-    ####^^^^^^^^####
+    z_offset_where_100mw_cm_2_in_mm = initial_offsets["100mw_calibration"]
 
     class Config:
         arbitrary_types_allowed = True
@@ -115,7 +110,11 @@ class Controller(BaseModel):
     def move_to_transit_z(self):
         self.move_z(self.z_max - self.z_abs_min)
 
+    def move_close_to_home_quick(self):
+        self.move_z(self.z_max, speed=400)
+
     def home_z(self):
+        self.move_close_to_home_quick()
         self._hardware.home_z(self.mount)
 
     def move_x(self, x_position: float, speed: float = 20):
@@ -134,20 +133,25 @@ class Controller(BaseModel):
         self._hardware.move_to(self.mount, position, speed=speed)
 
     def turn_on_laser(self, seconds_to_off: int = None):
+        make_sure_port_is_started()
+        self.protocol.comment(f"Turning on Laser for {seconds_to_off} seconds")
         turn_on_laser()
         if seconds_to_off is not None:
             time.sleep(seconds_to_off)
             turn_off_laser()
 
-    def move_to_well(self, well: Well, speed: float = 20):
+    def move_to_well(self, well: Well, height_of_liquid: float = 0, speed: float = 20):
+        make_sure_port_is_started()
         self.move_to_transit_z()
         p = well.top().point
+        self.protocol.comment(f"Moving Laser to {str(Well.display_name)}")
         adjusted_point = Point(x=p.x + self.x_offset, y=p.y + self.y_offset, z=self.get_current_pos().z)
-        self._move(adjusted_point, speed=speed)
-        self.move_z(p.z + self.z_offset_where_100mw_cm_2_in_mm, speed=speed)
 
-    def move_close_to_home_quick(self):
-        self.move_z(self.z_max, speed=600)
+        self._move(adjusted_point, speed=speed)
+
+        p_bottom_z = well.bottom().point.z
+
+        self.move_z(p.z + self.z_offset_where_100mw_cm_2_in_mm - p_bottom_z + height_of_liquid, speed=speed)
 
     def _print_pos(self):
         if self.print_debug_messages:
